indexing
	description: "Program points at exit from specific routine (associated with postconditions)"
	author: "Nadia Polikarpova"
	date: "$Date$"
	revision: "$Revision$"

class
	CI_ROUTINE_EXIT

inherit
	CI_ROUTINE_POINT
		redefine
			suppressor,
			ensures_invariant,
			add_variables,
			state_printing_function_formals,
			state_printing_function_actuals,
			instrument
		end

	CI_RUNTIME_OUTPUT_NAMES_CLIENT

create
	make

feature -- Access
	kind (a_visitor: CI_PROGRAM_POINT_KIND_FORMAT): STRING is
			-- Program point kind name
		do
			if is_function_point then
				Result := a_visitor.function_exit_kind
			else
				Result := a_visitor.exit_kind
			end
		end

	suppressor: CI_PROGRAM_POINT is
			-- Program point, who's assertions suppress assertions at current program point (Void if no suppressor)
		do
			if not is_secret (surrounding_routine) or is_exported_creation (surrounding_routine, universe) then
				Result := create {CI_STABLE_TIME}.make (surrounding_class, universe)
			end
		end

feature -- Status report
	ensures_invariant: BOOLEAN is
			-- Should invariant of instrumented class hold at program point?
		do
			Result := not is_secret (surrounding_routine) or is_exported_creation (surrounding_routine, universe)
		end

feature -- Assertions
	checked_assertions: ET_ASSERTIONS is
			-- Eiffel assertions, that are checked at this program point
		do
			Result := surrounding_routine.postconditions
		end

	instrument is
			-- Instrument program point by creating a check inside `surrounding_routine' with call to `state_printing_function'
		local
			internal_routine: ET_INTERNAL_ROUTINE
			check_instr: ET_CHECK_INSTRUCTION
		do
			internal_routine ?= surrounding_routine
			-- Works only for internal routine as present (more correctly is to call `state_printing_function' from check on client side)
			if internal_routine /= Void then
				create check_instr.make_with_capacity (1)
				check_instr.put_first (create {ET_CALL_EXPRESSION}.make (Void, state_printing_function_name, state_printing_function_actuals))
				insert_instruction_to_back (internal_routine.compound, check_instr)
			end
		end

feature {NONE} -- Assertions
	create_assertions is
			-- Create empty `checked_assertions', if they don't exist
		local
			list: ET_POSTCONDITIONS
		do
			if surrounding_routine.postconditions = Void then
				create list.make
				list.set_then_keyword (tokens.then_keyword)
				surrounding_routine.set_postconditions (list)
			end
		end

	process_original_assertions is
			-- Process original assertions according to settings while annotating this point
		local
			flat_post: ET_ASSERTIONS
		do
			if Settings.keep_original_contracts = Settings.remove_mode then
				checked_assertions.wipe_out
			elseif Settings.keep_original_contracts = Settings.flatten_mode then
				flat_post := flat_postcondition (surrounding_routine, surrounding_class, universe)
				checked_assertions.wipe_out
				prepend_assertions (checked_assertions, flat_post)
			end
		end

	state_printing_function_name: ET_FEATURE_NAME is
			-- Name of `state_printing_function' for this program point
		do
			create {ET_IDENTIFIER} Result.make (surrounding_class.lower_name + "_" +
				identifier_name (surrounding_routine.name).lower_name + "_post_")
		end

	state_printing_function_formals: ET_FORMAL_ARGUMENT_LIST is
			-- Formal arguments of `state_printing_function'
		do
			Result := Precursor
			if is_function_point then
				if Result = Void then
					create Result.make_with_capacity (1)
				end
				Result.force_first (create {ET_FORMAL_ARGUMENT}.make (corrupted (tokens.result_keyword.text),
					surrounding_function.declared_type))
			end
		end

	state_printing_function_actuals: ET_ACTUAL_ARGUMENT_LIST is
			-- Actual arguments of `state_printing_function' call inside instrumented `checked_assertions'
		do
			Result := Precursor
			if is_function_point then
				if Result = Void then
					create Result.make_with_capacity (1)
				end
				Result.force_first (tokens.result_keyword)
			end
		end

feature {NONE} -- Implementation
	add_variables is
			-- Find variables in scope and store them into `variables'
			-- Add local entities to variables found by `Precursor'
		local
			res: CI_RESULT
		do
			Precursor
			if is_function_point then
				create res.make (surrounding_function, surrounding_class, universe)
				-- Add `surrounding_class' to Result's clients
				res.clients.force_first (surrounding_class.name)
				variables.put (res)
			end
		end
end
