indexing
	description: "Expanded type kinds"
	author: "Nadia Polikarpova"
	date: "$Date$"
	revision: "$Revision$"

class
	CI_EXPANDED

inherit
	CI_VARIABLE_TYPE_KIND
		rename
			make as make_with_universe
		export {NONE}
			make_with_universe
		redefine
			is_expanded
		end

	CI_IDENTITIES_NAMES_CLIENT

create
	make

feature --Initialization
	make (a_class_name: like class_name; a_surrounding_class_name: like surrounding_class_name; a_universe: like universe) is
			-- Create expanded type kind for a variable with base class called `a_class_name'
			-- that appears in surrounding class called `a_surrounding_class_name' within `a_universe'
		require
			a_class_name_exists: a_class_name /= Void
			a_surrounding_class_name_exists: a_surrounding_class_name /= Void
			a_universe_exists: a_universe /= Void
		do
			make_with_universe (a_universe)
			surrounding_class_name := a_surrounding_class_name
			class_name := a_class_name
		ensure
			class_name_set: class_name = a_class_name
			surrounding_class_name_set: surrounding_class_name = a_surrounding_class_name
			universe_set: universe = a_universe
		end

feature -- Access
	identity (a_variable: CI_VARIABLE): ET_EXPRESSION is
			-- Expression with printable result that identifies value of expanded variable `a_variable'
		do
			create {ET_CALL_EXPRESSION} Result.make (
				create {ET_CALL_EXPRESSION}.make (Void, Identities_names.identity_target_name (surrounding_class_name), Void),
				Identities_names.identity_name (class_name),
				single_actual (a_variable.as_expression))
		end

	uniform_identity (an_expression: ET_IDENTIFIER): ET_EXPRESSION is
			-- Identity expression that has the same type (`INTEGER') independently of `base_type'
		do
			create {ET_CALL_EXPRESSION} Result.make (Void,
				Identities_names.identity_name (class_name),
				single_actual (an_expression))
		end

	uniform_identity_argument_type: ET_TYPE is
			-- Reference type, which is a valid "argument" type for `uniform_identity'
		once
			Result := universe.any_type
		end

	representation_type (a_class_name: ET_CLASS_NAME): STRING is
			-- Name of Daikon representation type for expanded type based on class called `a_class_name'
		once
			Result := hashcode_type
		end

	class_name: ET_CLASS_NAME
			-- Variable base class name (to be used in `uniform_identity' expression)

	surrounding_class_name: ET_CLASS_NAME
			-- Name of class, where variable is used (to be used in `identitiy' expression)
			-- Can be set to *UNKNOWN*, if `identity' expression is not used

feature -- Setting
	set_class_name (a_class_name: like class_name) is
			-- Set `class_name' to `a_class_name'
		require
			a_class_name_exists: a_class_name /= Void
		do
			class_name := a_class_name
		ensure
			class_name_set: class_name = a_class_name
		end

feature -- Status report
	is_expanded: BOOLEAN is
			--
		once
			Result := True
		end

invariant
	class_name_exists: class_name /= Void
	instrumented_class_name_exists: surrounding_class_name /= Void
end
